#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include "tadv.h"

int sadd_sketch(struct sketch *, struct sketch *);
struct sketch *find_sketch(char *);
struct sketch *bfind_sketch(struct sketch *, char *);
static void book_fail(char *);

struct sketch *book=NULL; // starting point

struct sketch *add_sketch(char *tag, char *text, struct option *opt)
{
    struct sketch *r, *s;
    if ((r=(struct sketch *)malloc(sizeof(struct sketch))) == NULL)
        book_fail("add_sketch() malloc");
    r->name = tag;
    r->text = text;
    r->opts = opt;
    r->right = NULL;
    r->left = NULL;

    if (book==NULL) book=r;
    else {
        sadd_sketch(book, r);
        /* linear structure
        for (s=book; s->right != NULL; s = s->right)
            ;
        s->right = r;
        */
    }
    return r;
}

int sadd_sketch(struct sketch *root, struct sketch *this)
{
    int cmp=strcmp(root->name, this->name);
    if (cmp > 0) {
        if (root->right == NULL) {
            root->right = this;
        } else {
            sadd_sketch(root->right, this);
        }
    } else if (cmp < 0) {
        if (root->left == NULL) {
            root->left = this;
        } else {
            sadd_sketch(root->left, this);
        }
    }
    return cmp;
}


struct option *add_option(char *mesg, struct action *act)
{
    struct option *r;
    if ((r=(struct option *)malloc(sizeof (struct option))) == NULL)
        book_fail("add_option() malloc");
    r->mesg = mesg;
    r->action = act;
    r->next = NULL;
    return r;
}


struct action *create_action(int cmd, char *tag)
{
    struct action *r;
    if ((r=(struct action *)malloc(sizeof (struct action))) == NULL)
        book_fail("create_action() malloc");
    r->command = cmd;
    switch (cmd) {
      case CMD_GOTO:
        if (tag==NULL)
            return NULL;
        else
           r->to = tag;
        break;
      case CMD_END:
        r->to = tag;
        break;
      default:
        return NULL;
    }
    return r;
}

struct sketch *find_sketch(char *name)
{
    return bfind_sketch(book, name);
    /* linear structure
    struct sketch *s;
    for (s = book; s != NULL; s=s->right) {
        if (strncmp(name, s->name, strlen(name)) == 0)
            return s;
    }
    return (NULL);
    */
}


struct sketch *bfind_sketch(struct sketch *root, char *name)
{
    if (!root)
        return (NULL);
    int cmp=strcmp(root->name, name);
    if (cmp > 0) {
        if (root->right == NULL) {
            return (NULL);
        } else {
            return bfind_sketch(root->right, name);
        }
    } else if (cmp < 0) {
        if (root->left == NULL) {
            return (NULL);
        } else {
           return bfind_sketch(root->left, name);
        }
    }
    return (root);
}


char *strcreate(char *s)
{
    char *r;
    if ((r=(char *)malloc(strlen((const char *)s)+1)) == NULL)
        book_fail("strcreate() malloc");
    strncpy(r, (const char *)s, strlen((const char *)s));
    return r;
}

static void book_fail(char *mesg)
{
    fprintf(stderr, "book.c:%s\n", mesg);
    exit(1);
}

